import { Emoji, GuildTextableChannel, Member, Message } from 'eris';
import { Client, Event } from '../class';

export default class MessageReactionAdd extends Event {
  public client: Client;

  constructor(client: Client) {
    super(client);
    this.event = 'messageReactionAdd';
  }

  public async run(message: Message<GuildTextableChannel>, _emoji: Emoji, reactor: Member) {
    if (message.channel.id !== '807444198969835550') return;
    if (message.author.id !== this.client.user.id) return;

    if (!reactor.roles[0]) {
      reactor = await message.channel.guild.getRESTMember(reactor.id);
    }

    if (!reactor.roles.includes('662163685439045632')) return;

    if ((await this.client.db.Proclamation.exists({ msg: message.id, processed: false }))) {
      const yea = await message.getReaction(this.client.util.emojis.SUCCESS);
      const nay = await message.getReaction(this.client.util.emojis.ERROR);
      const present = await message.getReaction('🙋');
      const totalDirectors = 6;
      const proclamation = await this.client.db.Proclamation.findOne({ msg: message.id });
      const processed = totalDirectors === (yea.length + nay.length + present.length) || Date.now() - proclamation.at > 604800000;
      const absent = totalDirectors - (yea.length + nay.length + present.length);

      await this.client.db.Proclamation.updateOne({ msg: message.id }, {
        results: {
          yea: yea.length,
          nay: nay.length,
          present: present.length,
          absent,
        },
        processed,
      });
      const inTheMajority = yea.length > nay.length + present.length;

      if (processed) {
        const author = this.client.users.get(proclamation.issuer) || await this.client.getRESTUser(proclamation.issuer);

        if (inTheMajority) {
          await author.createMessage(`__**Proclamation Majority Vote Received**__\nThe Proclamation you created at Library of Code sp-us, titled **${proclamation.subject}** (\`${proclamation.oID}\`) received the majority vote.`);
          await message.channel.createMessage(`__**Proclamation Results**__\nProclamation issued by ${author.mention} **received** the majority vote. Proclamation ID: ${proclamation.oID}\n\n__Results:__\n**Yea:** ${yea.length}\n**Nay:** ${nay.length}\n**Present:** ${present.length}\n**Absent:** ${absent}`);
        } else {
          await author.createMessage(`__**Proclamation Majority Vote Lost**__\nThe Proclamation you created at Library of Code sp-us, titled **${proclamation.subject}** (\`${proclamation.oID}\`) lost the majority vote.`);
          await message.channel.createMessage(`__**Proclamation Results**__\nProclamation issued by ${author.mention} **lost** the majority vote. Proclamation ID: ${proclamation.oID}\n\n__Results:__\n**Yea:** ${yea.length}\n**Nay:** ${nay.length}\n**Present:** ${present.length}\n**Absent:** ${absent}`);
        }
      }

      await reactor.user.createMessage(`__**Vote Recorded**\nYour vote on the proclamation with ID \`${proclamation.id}\` at Library of Code sp-us was successfully recorded.`);
    }
  }
}
